==> jaguar_jwt-0.1.0.tar.gz: lib/src/claimset.dart <==
part of jaguar_jwt.src;

class JaguarJwtClaimSet extends JwtClaimSet {
  /// Subject to which the token is issued
  final String subject;

  /// Issuer of the token
  final String issuer;

  /// Services the subject can access with this token
  final List<String> audience;

  /// When the token was issued
  final DateTime issuedAt;

  /// When the token expires
  final DateTime expiry;

  /// Extra payload
  final Map<String, dynamic> payload;

  JaguarJwtClaimSet(this.subject, this.issuer, this.audience, this.payload,
      {DateTime expiry, DateTime issuedAt})
      : issuedAt = issuedAt ?? new DateTime.now(),
        expiry = expiry ?? new DateTime.now().add(new Duration(days: 1));

  JaguarJwtClaimSet.fromJson(Map json)
      : subject = json["sub"],
        issuer = json["iss"],
        audience = json["aud"],
        issuedAt = new DateTime.fromMillisecondsSinceEpoch(json["iat"] * 1000),
        expiry = new DateTime.fromMillisecondsSinceEpoch(json["exp"] * 1000),
        payload = json['payload'];

  static JaguarJwtClaimSet parse(Map json) =>
      new JaguarJwtClaimSet.fromJson(json);

  @override
  Set<ConstraintViolation> validate(
      JaguarJwtClaimSetValidationContext validationContext) {
    Set<ConstraintViolation> violations = new Set<ConstraintViolation>();

    if (subject is! String || subject.isEmpty) {
      violations.add(new ConstraintViolation("Bad subject"));
    }

    if (issuer != validationContext.expectedIssuer) {
      violations.add(new ConstraintViolation("Invalid issuer: \"$issuer\""));
    }

    if (audience is List<String>) {
      bool _isAudienceOk = false;
      for (String aud in audience) {
        if (validationContext.expectedAudience.contains(aud)) {
          _isAudienceOk = true;
          break;
        }
      }
      if (!_isAudienceOk) {
        violations.add(new ConstraintViolation("Invalid audience!"));
      }
    }

    final now = new DateTime.now();

    if (now.isBefore(issuedAt.subtract(validationContext.maxClockSkew))) {
      var diff = issuedAt.difference(now);
      violations
          .add(new ConstraintViolation("Issued in the future: $diff h:m:s"));
    }
    if (now.isAfter(expiry.add(validationContext.maxClockSkew))) {
      var diff = now.difference(expiry);
      violations.add(new ConstraintViolation("Expired: by $diff h:m:s"));
    }

    return violations;
  }

  Map toJson() {
    return {
      'sub': subject,
      'iss': issuer,
      'aud': audience,
      'iat': (issuedAt.millisecondsSinceEpoch ?? 0) ~/ 1000,
      'exp': (expiry.millisecondsSinceEpoch ?? 0) ~/ 1000,
      'payload': payload
    };
  }
}

class JaguarJwtClaimSetValidationContext extends JwtClaimSetValidationContext {
  final String expectedIssuer;
  final List<String> expectedAudience;
  final Duration maxClockSkew;

  JaguarJwtClaimSetValidationContext(this.expectedIssuer, this.expectedAudience,
      {Duration maxClockSkew: const Duration(seconds: 30)})
      : maxClockSkew = maxClockSkew,
        super(expiryTolerance: maxClockSkew);
}

class JaguarJwtValidationContext extends JwtValidationContext {
  JaguarJwtValidationContext(JwaSignatureContext signatureContext,
      JaguarJwtClaimSetValidationContext csvc)
      : super(signatureContext, csvc) {}
}

==> jaguar_jwt-0.1.1.tar.gz: lib/src/claimset.dart <==
part of jaguar_jwt.src;

/// JWT Claim set used by Jaguar
class JaguarJwtClaimSet extends JwtClaimSet {
  /// Subject to which the token is issued
  final String subject;

  /// Issuer of the token
  final String issuer;

  /// Services the subject can access with this token
  final List<String> audience;

  /// When the token was issued
  final DateTime issuedAt;

  /// When the token expires
  final DateTime expiry;

  /// Extra payload
  final Map<String, dynamic> payload;

  JaguarJwtClaimSet(this.subject, this.issuer, this.audience, this.payload,
      {DateTime expiry, DateTime issuedAt})
      : issuedAt = issuedAt ?? new DateTime.now(),
        expiry = expiry ?? new DateTime.now().add(new Duration(days: 1));

  /// Constructs claim set form given [Map]
  factory JaguarJwtClaimSet.fromJson(Map json) {
    final String subject = json["sub"];
    final String issuer = json["iss"];
    final List<String> audience = json["aud"];
    DateTime issuedAt;
    if (json["iat"] is int) {
      issuedAt = new DateTime.fromMillisecondsSinceEpoch(json["iat"] * 1000);
    }
    DateTime expiry;
    if (json["exp"] is int) {
      expiry = new DateTime.fromMillisecondsSinceEpoch(json["exp"] * 1000);
    }
    Map<String, dynamic> payload = json['payload'];

    return new JaguarJwtClaimSet(subject, issuer, audience, payload,
        issuedAt: issuedAt, expiry: expiry);
  }

  /// A sugar to create [JaguarJwtClaimSet] from json [Map]
  static JaguarJwtClaimSet parse(Map json) =>
      new JaguarJwtClaimSet.fromJson(json);

  /// Validates the claim set with given [JaguarJwtClaimSetValidationContext]
  @override
  Set<ConstraintViolation> validate(
      JaguarJwtClaimSetValidationContext validationContext) {
    Set<ConstraintViolation> violations = new Set<ConstraintViolation>();

    if (subject is! String || subject.isEmpty) {
      violations.add(new ConstraintViolation("Bad subject"));
    }

    if (issuer != validationContext.expectedIssuer) {
      violations.add(new ConstraintViolation("Invalid issuer: \"$issuer\""));
    }

    if (validationContext.expectedAudience.length != 0) {
      if (audience is List<String>) {
        bool _isAudienceOk = false;
        for (String aud in audience) {
          if (validationContext.expectedAudience.contains(aud)) {
            _isAudienceOk = true;
            break;
          }
        }
        if (!_isAudienceOk) {
          violations.add(new ConstraintViolation("Invalid audience!"));
        }
      } else {
        violations.add(new ConstraintViolation("Invalid audience!"));
      }
    }

    final now = new DateTime.now();

    if (now.isBefore(issuedAt.subtract(validationContext.maxClockSkew))) {
      var diff = issuedAt.difference(now);
      violations
          .add(new ConstraintViolation("Issued in the future: $diff h:m:s"));
    }
    if (now.isAfter(expiry.add(validationContext.maxClockSkew))) {
      var diff = now.difference(expiry);
      violations.add(new ConstraintViolation("Expired: by $diff h:m:s"));
    }

    return violations;
  }

  /// Converts claim set to Map
  Map toJson() {
    return {
      'sub': subject,
      'iss': issuer,
      'aud': audience,
      'iat': (issuedAt.millisecondsSinceEpoch ?? 0) ~/ 1000,
      'exp': (expiry.millisecondsSinceEpoch ?? 0) ~/ 1000,
      'payload': payload
    };
  }
}

class JaguarJwtClaimSetValidationContext extends JwtClaimSetValidationContext {
  /// Expected issuer
  final String expectedIssuer;

  /// Expected audience
  final List<String> expectedAudience;

  /// Max clock skew
  final Duration maxClockSkew;

  JaguarJwtClaimSetValidationContext(this.expectedIssuer, this.expectedAudience,
      {Duration maxClockSkew: const Duration(seconds: 30)})
      : maxClockSkew = maxClockSkew,
        super(expiryTolerance: maxClockSkew);
}

class JaguarJwtValidationContext extends JwtValidationContext {
  JaguarJwtValidationContext(JwaSignatureContext signatureContext,
      JaguarJwtClaimSetValidationContext csvc)
      : super(signatureContext, csvc) {}
}
